/////////////////////////////////////////////////////////////////////
//                          Main.c                                 //
//                                                                 //
//  Purpose of this programm / but du programme:                   //
//  -Driver for controlling via RS232 the O'leds 13X48 LEDs screen //
//  -Pilote logiciel pour controller l'cran O'leds de 13X48 LEDs  //
//  Via un port srie RS-232                                       //
//                                                                 //
// Author: 	Weber Matthieu     http://matthieu.weber.free.fr   //
//                                                                 //
// Hardware/Matriel: PIC18F252 @ 20 Mhz                           //
//                                                                 //
//  <<    Beta version (some functions may not work)    >>         //
//                                                                 //
/////////////////////////////////////////////////////////////////////

// THIS CODE COMPILE WITH SDCC open source C compiler

// Firmware information
#define VERSION "0.60"
#define DATE    ", 2 mars 2008"

// Constants
#ifndef TWO_PI
#define TWO_PI 6.2831853071
#endif

// Includes
#define __18F252
#include <pic18F252.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "eeprom.c"
#include "uartsdcc.c"

#pragma stack 0x200 100  // Keep a free memory zone to allow bootloader code be copied inside

#define XTAL 20000000    // Quartz frequency
#define MAX_X  32        // X Size of the LEDs screen
#define MAX_Y  13        // Y Size of the LEDs screen
#define MAX_ARG_LEN 32   // Max size of an argument and an intruction name

#define BRATE 9600       // RS232  baud rate
#define VALEURBRG (XTAL/16/BRATE)-1

// Return codes
#define ERR_ARGS_OUT_OF_RANGE -2
#define ERR_SYNTAX            -3
#define OK  0
#define OK2 1   // To avoid displaying 'OK'


// Aliases for I/O - Wiring configuration
#define L1        PORTBbits.RB7     // LEDs matrix line 1
#define L2        PORTBbits.RB6     // LEDs matrix line 2
#define L3        PORTBbits.RB5     // LEDs matrix line 3
#define L4        PORTBbits.RB4     // LEDs matrix line 4
#define L5        PORTBbits.RB3     // LEDs matrix line 5
#define L6        PORTBbits.RB2     // LEDs matrix line 6
#define L7        PORTBbits.RB1     // LEDs matrix line 7
#define L8        PORTBbits.RB0     // LEDs matrix line 8
#define L9        PORTAbits.RA0     // LEDs matrix line 9
#define L10       PORTAbits.RA1     // LEDs matrix line 10
#define L11       PORTAbits.RA2     // LEDs matrix line 11
#define L12       PORTAbits.RA3     // LEDs matrix line 12
#define L13       PORTAbits.RA4     // LEDs matrix line 13

#define RS1       PORTCbits.RC2     // Output enable LEDs 1 to 9
#define RS2       PORTCbits.RC1     // Output enable LEDs 10 to 18
#define RS3       PORTCbits.RC0     // Output enable LEDs 19 to 27
#define RS4       PORTAbits.RA5     // Output enable LEDs 28 to 32

#define clock     PORTCbits.RC5     // Columns addressing

#define SCL       PORTCbits.RC3     // I2C Clock
#define SCK       PORTCbits.RC4     // I2C Data

#define TX        PORTCbits.RC6     // RS-232 serial data out
#define RX        PORTCbits.RC7     // RS-232 serial data in

// Fuses configuration : Brown-out-reset and start delay timer activated
typedef unsigned int config;
code char at __CONFIG1H config1h = 0xFF & _OSC_HS_1H & _OSCS_OFF_1H;
code char at __CONFIG2L config2l = 0xFF & _PUT_ON_2L & _BODEN_ON_2L & _BODENV_2_7V_2L;
code char at __CONFIG2H config2h = 0xFF & _WDT_OFF_2H;
code char at __CONFIG3H config3h = 0xFF & _CCP2MUX_RB3_3H;
code char at __CONFIG4L config4l = 0xFF & _LVP_OFF_4L & _BACKBUG_OFF_4L & _STVR_OFF_4L;
code char at __CONFIG5L config5l = 0xFF & _CP_0_OFF_5L & _CP_1_OFF_5L;
code char at __CONFIG5H config5h = 0xFF & _CPD_OFF_5H & _CPB_OFF_5H;
code char at __CONFIG6L config6l = 0xFF & _WRT_0_OFF_6L & _WRT_1_OFF_6L;
code char at __CONFIG6H config6h = 0xFF & _WRTD_OFF_6H & _WRTB_OFF_6H & _WRTC_OFF_6H;
code char at __CONFIG7L config7l = 0xFF & _EBTR_0_OFF_7L & _EBTR_1_OFF_7L;
code char at __CONFIG7H config7h = 0xFF & _EBTRB_OFF_7H;

unsigned short int count;
unsigned int mem0[MAX_X+1], mem1[MAX_X+1],  mem2[MAX_X+1]; // Graphic memories (main and additional)
unsigned short dimmer = 0, command = 0;

// Define CMOS 4017 output sequence
unsigned char sequence[] = {6, 4, 1, 9, 7, 3, 2, 5, 8, 10, 14, 11, 16, 18, 13, 12, 15, 17, 23, 19, 20, 26, 22, 24, 21, 25, 27, 30, 29, 28, 32, 31};
unsigned int col;

// Internal system functions
short GET(unsigned int _chr, unsigned char _n);
void  DelayMs(unsigned int _ms);
char* itoa(unsigned int _a);
unsigned int atoi(char* _a);
unsigned short isnum(char* _a);
short Parse(const char* _input,
                     char* _command, 
                     char* _arg0, 
                     char* _arg1, 
                     char* _arg2, 
                     char* _arg3,
                     char* _arg4, 
                     char* _arg5);

// Graphic functions
void  init_display(void);
short Set_pixel(unsigned char _x, unsigned char _y, unsigned char _mode);
short Line(unsigned char _x, unsigned char _y, unsigned char _length, unsigned char _mode,  unsigned int _angle);
short Rectangle(unsigned char _x1, unsigned char _y1, unsigned char _x2, unsigned char _y2, unsigned char _mode);
short Circle(unsigned char _x, unsigned char _y, unsigned char _radius, unsigned char _mode);
short Dimmer(const unsigned short _dimmer);

// Graphical memories operations
short SwapMem(unsigned short _memA, unsigned short _memB);
short DupMem(unsigned short _memDest, unsigned short _memSrc);
short ClearMem(unsigned short _memA);

//-----------------------------------------------------------------------------

void main(void)
  {
  unsigned int i, k, l,sens_x, sens_y;
  char instruct[MAX_ARG_LEN];
  char     arg0[MAX_ARG_LEN];
  char     arg1[MAX_ARG_LEN];
  char     arg2[MAX_ARG_LEN];
  char     arg3[MAX_ARG_LEN];
  char     arg4[MAX_ARG_LEN];
  char     arg5[MAX_ARG_LEN];

  //wait for system to power up, then set up the chip to do what we want
  for(i = 32760; i > 0; i--);

  //configure io ports
  TRISA  = 0x00;  // Port A is all outputs (led lines driving and 1/4 Columns RS)
  TRISB  = 0x00;  // Port B is all outputs (led lines driving)
  TRISC  = 0x98;  //####################  0x07;  // Pin C0, C1 & C2 are outputs (led lines driving) C5 = clock out,  C3,C4,C6 & C7 are use for Serial buses
  ADCON0 = 0x00;  // ADC Offline
  ADCON1 = 0x06;  // All A port set as digital I/O

  setup_UART(B9600_20); // 9600 bauds @ 20 Mhz

  // Timer0 configuration and activation
  T0CONbits.T0PS2  = 0;  // 1/64 Timer 0 prescaler
  T0CONbits.T0PS1  = 1;  // 1/64 Timer 0 prescaler
  T0CONbits.T0PS0  = 0;  // 1/64 Timer 0 prescaler
  T0CONbits.T08BIT = 1; // 8-bit mode
  T0CONbits.PSA    = 0; // prescaler enabled
  T0CONbits.T0CS   = 0; // internal clock source
  T0CONbits.TMR0ON = 1; // start timer

  INTCONbits.GIE = 1;   // enable interrupts globally
  //wait for system to power up, then set up the chip to do what we want
  for(i = 32760; i > 0; i--);

  RCONbits.IPEN = 0;            //disable priority levels
  INTCONbits.PEIE = 1;

  count = 1;         // Reset externals counters (CD 4017)
  clock   = 0;

  init_display();    // Clear matrix display

  // Switch foregrount text in white
  putc(0x1b);
  print_str("[37m");

  // Switch background into black (like in console mode)
  putc(0x1b);
  print_str("[40m");

  // Erase Terminal screen to display colors properly
  putc(0x1b);
  print_str("[2J");

  print_str("\n\r\n\rECRAN O'LED INITIALISE\n\r");    // write a welcome string on the UART
  print_str("\n\rTapez [aide] [help] ou [?] pour obtenir la liste des commandes\n\r");
  print_str("\n\r\n\>");


k =1;
l = 1;
sens_x = 1;
sens_y = 1;
init_display();

clear_buffer();  // Erase garbage RS232 data

INTCONbits.T0IE  = 1;   // enable timer 0 interrupt

while(1)
  {
  DelayMs(150);  DelayMs(150);DelayMs(150);DelayMs(150);DelayMs(150);DelayMs(150);
  DelayMs(150);  DelayMs(150);DelayMs(150);DelayMs(150);DelayMs(150);DelayMs(150);


  // Commands parser module
  if (command == 1)
    {
    unsigned short o = 0, status = ERR_SYNTAX;
    while(rx[o] != '\r') o++;
    rx[o] = '\0';

  // Erase Command syntax and args
  for (i = 0; i <= MAX_ARG_LEN; i++)
    {
    instruct[i] = '\0';
    arg0[i]     = '\0';
    arg1[i]     = '\0';
    arg2[i]     = '\0';
    arg3[i]     = '\0';
    arg4[i]     = '\0';
    arg5[i]     = '\0';
    }

    Parse(rx, instruct, arg0, arg1, arg2, arg3, arg4, arg5);

    if (!strcmp(instruct, "help") || !strcmp(instruct, "h") || !strcmp(instruct, "?") || !strcmp(instruct, "aide"))
      {
      print_str("\n\rhelp ou h ou ?                  : Affiche cet ecran d'aide");
      print_str("\n\rversion                         : Version courante du firmware");
      print_str("\n\rdimmer [niveau luminosite]      : Ajuste la luminosite de l'ecran");
      print_str("\n\rcls                             : Efface entierement l'ecran");

      print_str("\n\rpix  [x] [y] [etat]             : Allume/eteint un pixel");
      print_str("\n\rrec  [x1] [y1] [x2] [y2] [etat] : Dessine/efface un rectangle");
      print_str("\n\rcir  [x] [y] [rayon] [etat]     : Dessine/efface un cercle");

      print_str("\n\r([ETAT]: 0 => Eteint ; 1 => Allume ; 2 => Inverse ; 3 => Remplis ; 4 => Vide)\n\r");

      print_str("\n\rswapmem  [num. mem1] [num. mem2]: Echange le contenu de 2 memoires graphiques");
      print_str("\n\rdupmem   [num. mem1] [num. mem2]: Clone le contenu d'une memoire graphique");
      print_str("\n\rclearmem [num. mem]             : Efface le contenu d'une memoire graphique");
      status = OK2;
      }
    if (!strcmp(instruct, "ver") || !strcmp(instruct, "version") || !strcmp(instruct, "vrs") || !strcmp(instruct, "vers"))
      {
      print_str("\n\rOLED firmware version ");
      print_str(VERSION);
      print_str(" - (c) http://Matthieu.Weber.free.fr");
      print_str(DATE);
      status = OK2;
      }

    // Draw/erase a pixel
    if (!strcmp(instruct, "pix") || !strcmp(instruct, "pixel"))     {status = Set_pixel(atoi(arg0), atoi(arg1), atoi(arg2));}

    // Draw/erase a rectangle
    if (!strcmp(instruct, "rec") || !strcmp(instruct, "rectangle")) {status = Rectangle(atoi(arg0), atoi(arg1), atoi(arg2), atoi(arg3), atoi(arg4));}

    // Draw/erase a circle
    if (!strcmp(instruct, "cir") || !strcmp(instruct, "circle") || !strcmp(instruct, "cercle"))  {status = Circle(atoi(arg0), atoi(arg1), atoi(arg2), atoi(arg3));}

    // Clear the screen
    if (!strcmp(instruct, "cls") || !strcmp(instruct, "clrscr"))    {status = ClearMem(0);}

    // Adjust LEDs intensity
    if (!strcmp(instruct, "dimmer")) {status = Dimmer(atoi(arg0));}

    // Swaps between 2 graphic memories
    if (!strcmp(instruct, "swapmem"))  {status = SwapMem(atoi(arg0), atoi(arg1));}

    // Copy a graphic memory to another
    if (!strcmp(instruct, "dupmem"))   {status = DupMem(atoi(arg0), atoi(arg1));}

    // Erase a graphic memory
    if (!strcmp(instruct, "clearmem")) {status = ClearMem(atoi(arg0));}

    // ...If no command entered
    if (!strcmp(instruct, ""))  {print_str("\n\rType help or h or ? for commands list\n\r"); status = OK2;}

    // Syntax error
    if (status == ERR_SYNTAX)
      {
      print_str("\n\r");
      print_str(instruct);
      print_str(" : commande inconnue !");
      }

    // One or some arguments are out of range
    if (status == ERR_ARGS_OUT_OF_RANGE)
      {
      print_str("\n\r");
      print_str(rx);
      print_str(" : ATTENTION, un ou plusieurs arguments hors limites");
      }

    // Write the string OK
    if (status == OK)  print_str("\n\r OK");  // to say: action done !

    status = OK2; // reinit
    clear_buffer();
    command = 0;
    print_str("\n\r>"); // Displays prompt
    }
  }
}


//-----------------------------------------------------------------------------
// Clear matrix display
void init_display(void)
{
// Delete the 3 graphic memories
unsigned short _z;
for (_z = 0; _z < MAX_X+1; _z++)  
  {
  mem0[_z] = 0; mem1[_z] = 0; mem2[_z] = 0;
  }
}

//-----------------------------------------------------------------------------

void DelayMs(unsigned int _ms)
{
unsigned int z;
for (z = 0 ; z <= _ms ; z++);
for (z = 0 ; z <= _ms ; z++);
for (z = 0 ; z <= _ms ; z++);
for (z = 0 ; z <= _ms ; z++);
for (z = 0 ; z <= _ms ; z++);
}

//-----------------------------------------------------------------------------

// Interruptions management function
void voidisr(void)interrupt 1
{
  if (PIR1bits.RCIF == 1)        // We received a char on the UART
    {
    unsigned char caractere;

    caractere=RCREG;             // take received char
    if ((caractere > 31) && (caractere < 127))  putc(caractere);  // Echo on UART received char printable char (avoid arrows)
    rx[place] = caractere;
    place++;
    if (place >= BUFFER) place = 0;

    if (caractere == '\r')
      {
      putc('\n');                // Goto next line if pressed [ENTER]
      command = 1;               // New command arrived
      }

    PIR1bits.RCIF=0;             // clear RX interrupt flag
    }

  if (INTCONbits.T0IF == 1)      // Interruption from TIMER 0
    {
    clock = 0;

    L1  = 1;
    L2  = 1;
    L3  = 1;
    L4  = 1;
    L5  = 1;
    L6  = 1;
    L7  = 1;
    L8 = 1;L9  = 1;L10  = 1;L11  = 1;L12  = 1;L13  = 1;
    PORTA = 0xFF;   // A remettre ensuite (trouver bon dlais de pause...)

    clock = 1;

    RS4 = !( count > 26);
    RS3 = !((count < 27)&&(count > 17));
    RS2 = !((count < 19)&&(count > 8));
    RS1 = !( count < 10);

    col = mem0[sequence[count-1]];
    L1  = GET(col, 0);
    L2  = GET(col, 1);
    L3  = GET(col, 2);
    L4  = GET(col, 3);
    L5  = GET(col, 4);
    L6  = GET(col, 5);
    L7  = GET(col, 6);
    L8  = GET(col, 7);
    L9  = GET(col, 8);
    L10 = GET(col, 9);
    L11 = GET(col, 10);
    L12 = GET(col, 11);
    L13 = GET(col, 12);
    count++;

    if (count > MAX_X)  {count = 0;}
    }
  INTCONbits.T0IF = 0; // aquitement de l'interruption de dbordement du timer
}

//-----------------------------------------------------------------------------
// Test a pixel
short GET(unsigned int _chr, unsigned char _n)
{
  /*tester le n-ime bit*/
  if( (_chr & (1<<_n)) !=0)  return 0;
  return 1;
}

//-----------------------------------------------------------------------------

// Mode = 0  => Set pixel x y to OFF.
// Mode = 1  => Set pixel x y to ON.
// Mode = 2  => Invert pixel x y.
short Set_pixel(unsigned char _x, unsigned char _y, unsigned char _mode)
{
  if ((_x > MAX_X) || (_y > MAX_Y) || _mode > 3) return ERR_ARGS_OUT_OF_RANGE;

  if (_mode == 1)  {mem0[_x] = (mem0[_x] | (1<<(_y-1)));}          // Set pixel _x _y to ON
//  if (mode == 0) { mem0[x] = mem0[x] & (0xFFFF ^ (1<<y));}       // Set pixel _x _y to OFF
  if (_mode == 0)  {mem0[_x] = (mem0[_x] & ~(1<<(_y-1)));}         // Set pixel _x _y to OFF

  //  if (_mode == 2)  mem0[_x] = mem0[_x] & (1<<_y);             // pixel _x _y state is inverted
  return OK;
}

//-----------------------------------------------------------------------------

// Draw/erase/invert a line begining at (x1;y1) point. Possible angles are: 0 => horizontal,
// 45 => raising diagonal, 90 => vertical, 135 => lowering diagonal
// length => Size in pixels.
// Mode = 0  => Erase lined pixel.
// Mode = 1  => Draw a line.
// Mode = 2  => Invert rectangle border.
short Line(unsigned char _x, unsigned char _y, unsigned char _length, unsigned char _mode,  unsigned int _angle)
{
  unsigned short i;
  if ((_angle == 0)   || (_angle == 360)) {for (i = 0; i < _length; i++) Set_pixel(_x+i, _y,   _mode);}
  if ((_angle == 90)  || (_angle == 270)) {for (i = 0; i < _length; i++) Set_pixel(_x,   _y+i, _mode);}
  if ((_angle == 45)  || (_angle == 315)) {for (i = 0; i < _length; i++) Set_pixel(_x+i, _y+i, _mode);}
  if ((_angle == 135) || (_angle == 225)) {for (i = 0; i < _length; i++) Set_pixel(_x-i, _y-i, _mode);}

  if ((_x > MAX_X) || (_y > MAX_Y) || _mode > 3) return ERR_ARGS_OUT_OF_RANGE;
  else  return OK;
}

//-----------------------------------------------------------------------------

// Draw/erase/invert a rectangle between (x1;y1) (x2;y2) points.
// Mode = 0  => Erase rectangle border.
// Mode = 1  => Draw empty rectangle.
// Mode = 2  => Invert rectangle border.
// Mode = 3  => Erase rectangular aera.
// Mode = 4  => Draw filled rectangle.
// Mode = 5  => Invert rectangular aera.
short Rectangle(unsigned char _x1, unsigned char _y1, unsigned char _x2, unsigned char _y2, unsigned char _mode)
{
  unsigned short i, j;
  if (_mode < 3)
    {
    for (i = 0; i <= ((_x2-_x1)); i++) {Set_pixel(_x1+i, _y1, _mode);  Set_pixel(_x1+i, _y2, _mode);}
    for (i = 0; i <= ((_y2-_y1)); i++) {Set_pixel(_x1, _y1+i, _mode);  Set_pixel(_x2, _y1+i, _mode);}
    }
  else
    {
    for (i = 0; i <= (_x2-_x1); i++)
      {
      for (j = 0; j <= (_y2-_y1); j++)  Set_pixel(_x1+i, _y1+j, _mode - 3);
      }
    }
if ((_x1 > MAX_X) || (_y1 > MAX_Y) || (_x2 > MAX_X) || (_y2 > MAX_Y) || _mode > 5) return ERR_ARGS_OUT_OF_RANGE;
else  return OK;
}

//-----------------------------------------------------------------------------

// Draw/erase/invert a circle with center at (x1;y1), radius in pixels.
// Mode = 0  => Erase circle border.
// Mode = 1  => Draw empty circle.
// Mode = 2  => Invert circle border.
// Mode = 3  => Erase circular aera.
// Mode = 4  => Draw filled circle.
// Mode = 5  => Invert circular aera.
short Circle(unsigned char _x, unsigned char _y, unsigned char _radius, unsigned char _mode)
{
  unsigned short i, j;
  if (_mode < 3)
    {
    for (i = 0; i < 360; i++) {Set_pixel(_x+cosf(i/TWO_PI)*_radius, _y+sinf(i/TWO_PI)*_radius, _mode);}
    }
  else
    {
    for (i = 0; i < 360; i++)
      {
      for (j = 0; j < 360; j++)  Set_pixel(_x+i, _y+j, _mode - 3); //  #### A IMPLEMENTER
      }
    }
if ((_x > MAX_X) || (_y > MAX_Y) || _mode > 5) return ERR_ARGS_OUT_OF_RANGE;
else  return OK;
}

//-----------------------------------------------------------------------------

short Dimmer(const unsigned short _dimmer)
{

  if (_dimmer > 2)  return ERR_ARGS_OUT_OF_RANGE;

  T0CONbits.TMR0ON = 0; // stop timer
  INTCONbits.GIE = 0;   // disable interrupts globally
  switch (_dimmer)
    {
    case 0:
      {
      T0CONbits.T0PS2 = 0;  // 1/64 Timer 0 prescaler
      T0CONbits.T0PS1 = 1;  // 1/64 Timer 0 prescaler
      T0CONbits.T0PS0 = 0;  // 1/64 Timer 0 prescaler
      print_str("\n\rdimmer set to 0\n");
      break;
      }
    case 1:
      {
      T0CONbits.T0PS2 = 0;  // 1/64 Timer 0 prescaler
      T0CONbits.T0PS1 = 0;  // 1/64 Timer 0 prescaler
      T0CONbits.T0PS0 = 1;  // 1/64 Timer 0 prescaler
      print_str("\n\rdimmer set to 1\n");
      break;
      }
    case 2:
    default:
      {
      T0CONbits.T0PS2 = 0;  // 1/64 Timer 0 prescaler
      T0CONbits.T0PS1 = 0;  // 1/64 Timer 0 prescaler
      T0CONbits.T0PS0 = 0;  // 1/64 Timer 0 prescaler
      print_str("\n\rdimmer set to 2\n");
      break;
      }
    }
  T0CONbits.T08BIT = 1; // 8-bit mode
  T0CONbits.PSA    = 0; // prescaler enabled
    T0CONbits.T0CS   = 0; // internal clock source

T0CONbits.TMR0ON = 1; // start timer
  INTCONbits.GIE = 1;   // enable interrupts globally

return OK;
}

//-----------------------------------------------------------------------------

short Parse(const char* _input,
                  char* _instruct,
                  char* _arg0,
                  char* _arg1,
                  char* _arg2,
                  char* _arg3,
                  char* _arg4,
                  char* _arg5)
{
unsigned short z = 0, section = 0, size = 0;


while ((_input[z] != '\r') && (_input[z] != '\n') && (z < BUFFER))
  {
  // Trim and identify space char or tab char
  if ((_input[z] == ' ') || (_input[z] == '\t')) {z++; continue;}

  // Starting from here, we got printable chars
  if (z > 0)
    {
    if ((_input[z-1] == ' ') || (_input[z-1] == '\t'))
      {
      section++;
      size = 0;
      }
    }
  else{section++;}

  if (section == 1)  _instruct[size] = _input[z];
  if (section == 2)  _arg0[size]     = _input[z];
  if (section == 3)  _arg1[size]     = _input[z];
  if (section == 4)  _arg2[size]     = _input[z];
  if (section == 5)  _arg3[size]     = _input[z];
  if (section == 6)  _arg4[size]     = _input[z];
  if (section == 7)  _arg5[size]     = _input[z];
  if (section >  7)  return ERR_ARGS_OUT_OF_RANGE;
  size++;
  z++;
  }
return section;  // Function returns command & args numbers
}

//-----------------------------------------------------------------------------

short SwapMem(unsigned short _memA, unsigned short _memB)
{
// Swaps between 2 graphical memories (mem0 is always the displayed graphical memory)
unsigned short _z;
unsigned int tmp;
unsigned int* a;
unsigned int* b;

if (_memA > 2 || _memB > 2)  return ERR_ARGS_OUT_OF_RANGE;

a = mem0;  // default
if (_memA == 1) a = mem1;
if (_memA == 2) a = mem2;

b = mem0;  // default
if (_memB == 1) b = mem1;
if (_memB == 2) b = mem2;

for (_z = 0; _z < MAX_X+1; _z++)
  {
  tmp = a[_z];
  a[_z] = b[_z];
  b[_z] = tmp;
  }
return OK;
}

//-----------------------------------------------------------------------------

short DupMem(unsigned short _memDest, unsigned short _memSrc)
{
// Copy a graphical memory to another (mem0 is always the displayed graphical memory)
unsigned short _z;
unsigned int* a;
unsigned int* b;

if (_memDest > 2 || _memSrc > 2)  return ERR_ARGS_OUT_OF_RANGE;

a = mem0;  // default
if (_memDest == 1) a = mem1;
if (_memDest == 2) a = mem2;

b = mem0;  // default
if (_memSrc == 1) b = mem1;
if (_memSrc == 2) b = mem2;
for (_z = 0; _z < MAX_X+1; _z++)
  {
  a[_z] = b[_z];
  }
return OK;
}

//-----------------------------------------------------------------------------

// Delete a selected graphic memory
short ClearMem(unsigned short _memA)
{
unsigned short _z;
unsigned int* a;

if (_memA > 2)  return ERR_ARGS_OUT_OF_RANGE;

a = mem0;  // default
if (_memA == 1) a = mem1;
if (_memA == 2) a = mem2;

for (_z = 0; _z < MAX_X+1; _z++)  a[_z] = 0;
return OK;
}
//-----------------------------------------------------------------------------

/*
mettre le n-ieme bit a 0
mask = mask & ~(1<<n);
*/

//-----------------------------------------------------------------------------

int Atoi(const char *Str)
{
int R = 0;
short N = 1;
if(*Str == '-')
  {
  N = -1;
  ++Str;
  }
while(*Str != '\0' && *Str >= '0' && *Str <= '9')
  R = (R * 10) + (*Str++ - '0');
return (R * N);
}

//-----------------------------------------------------------------------------

// Convert a positive integer into a string
char * itoa(unsigned int _a)
{
#define BUFFSIZE 50

static char str[BUFFSIZE] = "                                 ";

int i = sizeof(str) - 1;

do
{
str[--i] = '0' + _a % 10;
}
while ((_a = _a / 10) && i >= 0);

return &str[i];
}

//-----------------------------------------------------------------------------

// Return true if the string _a contains a number; else return false
unsigned short isnum(char* _a)
{
unsigned short z = 0;
while ((_a[z] != '\n') && (_a[z] != '\r'))
  {
  z++;
  if ((_a[z] < 48) && (_a[z] > 57))
  return 0;
  }
return 1;
}
