//#include <common.h>
//#include <meters.h>
//#include <user.h>
//#include <stdbool.h>
#include <pic18F252.h>

// Routines to do the actual reading and writing of the internal eeprom.

// reads the eeprom at address a and returns the contents there
unsigned int eeprom_read4(unsigned short a){
	static	unsigned int r;
	static	unsigned short i;

	for (i = 0; i < 4; i++){
		EEADR = a + i;
		EECON1bits.EEPGD = 0;
		EECON1bits.CFGS = 0;
		EECON1bits.RD = 1;
		((unsigned short *)(&r))[i] = EEDATA;
	}
	return r;
}

// writes the byte x to the eeprom at address a
void eeprom_write4(unsigned int x,unsigned short a){
	static unsigned short i;

	for (i = 0; i < 4; i++){
		while (EECON1bits.WR);

		PIR2bits.EEIF = 0;
	
		EEADR = a + i;
		EEDATA = *((unsigned short *)(&x) + i);
		EECON1bits.EEPGD = 0;
		EECON1bits.CFGS = 0;
		EECON1bits.WREN = 1;

		// critical bit here
		_asm 
			movlw	0x55
			movwf	_EECON2
			movlw	0xAA
			movwf	_EECON2
			bsf	_EECON1,1 
		_endasm;
		while (EECON1bits.WR);
	}
}

// reads a 4-byte wide eeprom value, performing error correcting
unsigned int eeprom_read(unsigned short ad){
	static unsigned int a,b,c;

	a = eeprom_read4(ad);
	b = eeprom_read4(ad + 4);
	c = eeprom_read4(ad + 8);

	// return the majority winner

	if (a == b)
		return a; // write of c may have failed
	if (a == c)
		return a; // write of b failed
	if (b == c)
		return b; // write of a failed
	return c; // all failed, c, which is written last, is probably right

}

// writes a 4-byte wide eeprom value, with error checking
void eeprom_write(unsigned int x,unsigned short ad){
	static unsigned int a;
	
	eeprom_write4(x,ad);
	eeprom_write4(x,ad + 4);
	eeprom_write4(x,ad + 8);
	
	// verify, on failure try the write once
	a = eeprom_read4(ad);
	if (a != x)
		eeprom_write4(x,ad);
	a = eeprom_read4(ad + 4);
	if (a != x)
		eeprom_write4(x,ad + 4);
	a = eeprom_read4(ad + 8);
	if (a != x)
		eeprom_write4(x,ad + 8);

}

