// UART Library for PIC 18
// SDCC version / small device c compiler
// written by Jean-Pierre MANDON 2006 jp.mandon@free.fr
// www.pictec.org
/*
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   In other words, you are welcome to use, share and improve this program.
   You are forbidden to forbid anyone else to use, share and improve
   what you give them.   Help stamp out software-hoarding!
*/

// SPBRG value for FOSC 10 Mhz

#define B1200_10         0,129
#define B2400_10         0,64
#define B9600_10         1,64
#define B19200_10        1,32
#define B38400_10        1,16

// SPBRG value for FOSC 20 Mhz

#define B2400_20         0,129
#define B9600_20         1,129
#define B19200_20        1,64
#define B38400_20        1,32

// SPBRG value for FOSC 40 Mhz

#define B9600_40         0,64
#define B19200_40        1,129
#define B38400_40        1,64
#define B56000_40        1,44
#define B57600_40        1,42
#define B115200_40       1,21

// local definition UART.C

#define BUFFER 50                      // rx buffer length
char rx[BUFFER];                        // this is the buffer
unsigned char place = 0;

#define receivedchar wpointer!=rpointer

// setup PIC UART

void setup_UART(unsigned char _BRGH,unsigned char diviseur)
{
TXSTAbits.BRGH=_BRGH;                   // set BRGH bit
SPBRG=diviseur;                         // set UART speed SPBRG
RCSTA=0x90;                             // set RCEN and SPEN
PIE1bits.RCIE=1;                        // enable interrupt on RX
INTCONbits.PEIE=1;                      // enable peripheral interrupts
IPR1bits.RCIP=1;                        // define high priority for RX interrupt
//wpointer=1;                             // initialize write pointer
//rpointer=1;                             // initialize read pointer
TXSTAbits.TXEN=1;                       // enable TX
}

// UART write char

void putc(unsigned char caractere)
{
while (!TXSTAbits.TRMT);
// while (!PIR1bits.TXIF);	        // TX register empty ?
TXREG=caractere;		        // yes, send char
}

/*
char getch()
{
char caractere=0;

if (receivedchar)
    {
    caractere=rx[rpointer++];
    if (rpointer==BUFFER)
	rpointer=1;
    }
return(caractere);
}
*/
// clear rx buffer

void clear_buffer(void) 
{   unsigned short i;
    for (i = 0; i < BUFFER; i++) rx[i] = '\0';
    place = 0;
}

// write a string on the UART

void print_str(const char* text)
{
unsigned char i=0;
while ( text[i] != 0 )
    putc( text[i++] );
}

// convert 8 bits binary value and send it on the UART

void print_hex8(unsigned char valeur)
{
  static const char hexa[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
  static unsigned char index;
index=(valeur>>4)& 0x0F;
putc(hexa[index]);
index=valeur&0x0F;
putc(hexa[index]);
}

